/*
 * The contents of this file are subject to the terms 
 * of the Common Development and Distribution License 
 * (the "License").  You may not use this file except 
 * in compliance with the License.
 * 
 * You can obtain a copy of the license at 
 * glassfish/bootstrap/legal/CDDLv1.0.txt or 
 * https://glassfish.dev.java.net/public/CDDLv1.0.html. 
 * See the License for the specific language governing 
 * permissions and limitations under the License.
 * 
 * When distributing Covered Code, include this CDDL 
 * HEADER in each file and include the License file at 
 * glassfish/bootstrap/legal/CDDLv1.0.txt.  If applicable, 
 * add the following below this CDDL HEADER, with the 
 * fields enclosed by brackets "[]" replaced with your 
 * own identifying information: Portions Copyright [yyyy] 
 * [name of copyright owner]
 */

package com.sun.enterprise.web.connector.grizzly.async;

import com.sun.enterprise.web.connector.grizzly.AsyncExecutor;
import com.sun.enterprise.web.connector.grizzly.AsyncFilter;
import com.sun.enterprise.web.connector.grizzly.AsyncHandler;

import java.util.concurrent.Callable;
import java.util.concurrent.ScheduledThreadPoolExecutor;
import java.util.concurrent.TimeUnit;

/**
 * Simple implementation to demonstrate the asynchronous request processing. 
 * The <code>ProcessorTask</code> will be delayed by 10 seconds and then executed.
 *
 * @author Jeanfrancois Arcand
 */
public class DelayedAsyncFilter implements AsyncFilter, Callable {
    
    private final static ScheduledThreadPoolExecutor scheduler 
            = new ScheduledThreadPoolExecutor(1);    
    
    private AsyncHandler asyncHandler;
    
    private AsyncProcessorTask asyncProcessorTask;
    
    public boolean doFilter(AsyncExecutor asyncExecutor) {
        asyncProcessorTask = (AsyncProcessorTask)asyncExecutor.getAsyncTask();
        asyncHandler = asyncExecutor.getAsyncHandler();
        
        scheduler.schedule(this,10,TimeUnit.SECONDS);
        return false;
    }
    
    public Object call() throws Exception{   
        asyncProcessorTask.getProcessorTask().invokeAdapter();
        asyncHandler.handle(asyncProcessorTask);
        return null;
    }   
}
