/*
 * The contents of this file are subject to the terms 
 * of the Common Development and Distribution License 
 * (the "License").  You may not use this file except 
 * in compliance with the License.
 * 
 * You can obtain a copy of the license at 
 * glassfish/bootstrap/legal/CDDLv1.0.txt or 
 * https://glassfish.dev.java.net/public/CDDLv1.0.html. 
 * See the License for the specific language governing 
 * permissions and limitations under the License.
 * 
 * When distributing Covered Code, include this CDDL 
 * HEADER in each file and include the License file at 
 * glassfish/bootstrap/legal/CDDLv1.0.txt.  If applicable, 
 * add the following below this CDDL HEADER, with the 
 * fields enclosed by brackets "[]" replaced with your 
 * own identifying information: Portions Copyright [yyyy] 
 * [name of copyright owner]
 */
package com.sun.enterprise.web.connector.grizzly.comet;

import com.sun.enterprise.web.connector.grizzly.comet.CometHandler;
import java.io.IOException;
import java.io.PrintWriter;
import javax.servlet.ServletConfig;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;


/**
 * Simple servlet that use Grizzly Asynchronous Request Processing to fecth
 * mail from an email account (Gmail, Yahoo, etc.). The Servlet will be interupted 
 * until new mails are available in the Gmail account.
 *
 * @author Jeanfrancois Arcand
 */
public class CometServlet extends HttpServlet{
    
    private String contextPath;
     
    public CometServlet() {
    }

    /**
     * Init this Servlet by registering it with the 
     * <code>JavaMailAsyncFilter</code>.
     */
    public void init(ServletConfig config) throws ServletException { 
        super.init(config);
        contextPath = config.getServletContext().getContextPath() 
            + "/CometServlet";
        CometEngine cometEngine = CometEngine.getEngine();
        CometContext context = cometEngine.register(contextPath);    
        context.setExpirationDelay(10 * 1000);
    }
    
    
    protected void service(HttpServletRequest request, HttpServletResponse response) 
                                        throws ServletException, IOException {
        
        String action = request.getParameter("action");
        if (action != null) {
            if ("login".equals(action)) {
                String nickname = request.getParameter("nickname");
                request.getSession(true).setAttribute("nickname", nickname);
                response.sendRedirect("post.jsp");
            } else {
                String nickname = (String) request.getSession(true).getAttribute("nickname");
                String message = request.getParameter("message");
                //messageSender.send(nickname, message);
                response.sendRedirect("post.jsp");
            }
        } else {
        
            PrintWriter out = response.getWriter();
            try{
                out.println("Hello from doGet");
                out.flush();

                CometRequestHandler handler = new CometRequestHandler();
                handler.attach(response);

                CometEngine cometEngine = CometEngine.getEngine();
                CometContext cometContext = cometEngine.getCometContext(contextPath);
                cometContext.addCometHandler(handler);
                cometContext.notify("HelloFromMe");
            } catch (Throwable t){
               t.printStackTrace(); 
            }
        }
    }

    // --------------------------------------------------------- Async Hook ---/

    public String getContextPath() {
        return contextPath;
    }

    
    public class CometRequestHandler implements CometHandler<HttpServletResponse>{
        
        private HttpServletResponse httpServletResponse;
        
        public void attach(HttpServletResponse httpServletResponse){
            this.httpServletResponse = httpServletResponse;
        }

                
        public void onEvent(CometEvent event) throws IOException{   
            System.out.println("==== onEvent =====");
            try{
                PrintWriter printWriter = httpServletResponse.getWriter();
                printWriter.println("Hello from onEvent from Servlet: ");
                printWriter.println("====" + event.attachment() + "======");
                printWriter.flush();
            } catch (Throwable t){
               t.printStackTrace(); 
            }  
        }


        public void onInitialize(CometEvent event) throws IOException{  
        }


        public void onTerminate(CometEvent event) throws IOException{
            onInterrupt(event);
        }


        public void onInterrupt(CometEvent event) throws IOException{
            CometEngine cometEngine = CometEngine.getEngine();
            CometContext cometContext = 
                    cometEngine.getCometContext(contextPath);    
            cometContext.removeCometHandler(this);
        }        
    }
}
